/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_TRPCLATSERVICE_TRPCLATSERVICE_PROXY_H
#define JAC_AP_TRPCLATSERVICE_TRPCLATSERVICE_PROXY_H

#include "ara/com/internal/proxy/proxy_adapter.h"
#include "ara/com/internal/proxy/event_adapter.h"
#include "ara/com/internal/proxy/field_adapter.h"
#include "ara/com/internal/proxy/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/trpclatservice/trpclatservice_common.h"
#include <string>

namespace jac {
namespace ap {
namespace trpclatservice {
namespace proxy {
namespace events {
    using TRPCLat2CANEvent = ara::com::internal::proxy::event::EventAdapter<::dpc::SWC2CAN_Signal_Values>;
    using Lat2CANEvent = ara::com::internal::proxy::event::EventAdapter<::SWC2CAN>;
    static constexpr ara::com::internal::EntityId TRPCLatServiceTRPCLat2CANEventId = 30378U; //TRPCLat2CANEvent_event_hash
    static constexpr ara::com::internal::EntityId TRPCLatServiceLat2CANEventId = 36711U; //Lat2CANEvent_event_hash
}

namespace fields {
}

namespace methods {

} // namespace methods

class TRPCLatServiceProxy {
private:
    std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
public:
    using HandleType = ara::com::HandleType;
    class ConstructionToken {
    public:
        explicit ConstructionToken(const std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> & proxy)
            : proxyAdapter(proxy),
              TRPCLat2CANEvent(proxyAdapter->GetProxy(), events::TRPCLatServiceTRPCLat2CANEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              Lat2CANEvent(proxyAdapter->GetProxy(), events::TRPCLatServiceLat2CANEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

        ConstructionToken(ConstructionToken&& other) = default;
        ConstructionToken& operator=(ConstructionToken && other) = default;

        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        
        virtual ~ConstructionToken() = default;

        /* Do not use such internal interfaces!!! */
        ara::core::Result<void> InitializeAll()
        {
            ara::core::Result<void> initResult;
            return initResult;
        }
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> GetProxyAdapter()
        {
            return proxyAdapter;
        }
        events::TRPCLat2CANEvent GetTRPCLat2CANEvent() noexcept { return std::move(TRPCLat2CANEvent); }
        events::Lat2CANEvent GetLat2CANEvent() noexcept { return std::move(Lat2CANEvent); }
    private:
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
        events::TRPCLat2CANEvent TRPCLat2CANEvent;
        events::Lat2CANEvent Lat2CANEvent;
    };

    virtual ~TRPCLatServiceProxy()
    {
        TRPCLat2CANEvent.UnsetReceiveHandler();
        TRPCLat2CANEvent.Unsubscribe();
        Lat2CANEvent.UnsetReceiveHandler();
        Lat2CANEvent.Unsubscribe();

    }

    explicit TRPCLatServiceProxy(const HandleType &handle)
        : proxyAdapter(std::make_shared<ara::com::internal::proxy::ProxyAdapter>(::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, handle)),
          TRPCLat2CANEvent(proxyAdapter->GetProxy(), events::TRPCLatServiceTRPCLat2CANEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          Lat2CANEvent(proxyAdapter->GetProxy(), events::TRPCLatServiceLat2CANEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

    void ThrowError(const ara::core::Result<void>& result) const
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create proxy failed!"<< std::endl;
#endif
        }
    }

    TRPCLatServiceProxy(const TRPCLatServiceProxy&) = delete;
    TRPCLatServiceProxy& operator=(const TRPCLatServiceProxy&) = delete;

    TRPCLatServiceProxy(TRPCLatServiceProxy&&) = default;
    TRPCLatServiceProxy& operator=(TRPCLatServiceProxy&&) = default;
    TRPCLatServiceProxy(ConstructionToken&& token) noexcept
        : proxyAdapter(token.GetProxyAdapter()),
          TRPCLat2CANEvent(token.GetTRPCLat2CANEvent()),
          Lat2CANEvent(token.GetLat2CANEvent()){
        static_cast<void>(std::move(token));
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        const ara::com::HandleType &handle)
    {
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> preProxyAdapter =
            std::make_shared<ara::com::internal::proxy::ProxyAdapter>(
               ::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, handle);
        ConstructionToken token{std::move(preProxyAdapter)};
        ara::core::Result<void> initResult {token.InitializeAll()};
        if (initResult.HasValue()) {
            return ara::core::Result<ConstructionToken>(std::move(token));
        } else {
            const ara::core::ErrorCode errorcode{initResult.Error()};
            ara::core::Result<TRPCLatServiceProxy::ConstructionToken> preResult{errorcode};
            return preResult;
        }
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType>& handler,
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, instance);
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType> handler,
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, specifier);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, instance);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::trpclatservice::TRPCLatService::ServiceIdentifier, specifier);
    }

    static void StopFindService(const ara::com::FindServiceHandle& handle)
    {
        ara::com::internal::proxy::ProxyAdapter::StopFindService(handle);
    }

    HandleType GetHandle() const
    {
        return proxyAdapter->GetHandle();
    }
    bool SetEventThreadNumber(const std::uint16_t number, const std::uint16_t queueSize)
    {
        return proxyAdapter->SetEventThreadNumber(number, queueSize);
    }
    events::TRPCLat2CANEvent TRPCLat2CANEvent;
    events::Lat2CANEvent Lat2CANEvent;
};
} // namespace proxy
} // namespace trpclatservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_TRPCLATSERVICE_TRPCLATSERVICE_PROXY_H
